"""Compute sums of beta-bernoulli variables

Functions
---------
bbs_pmf
    Probability mass function for BBS
bbs_cdf
    Cumulative distribution function for BBS
bbs_test
    Hypothesis test for BBS
"""

from betabernsum.bbs import bbs_pmf, bbs_cdf
from betabernsum.bbs_test import bbs_test